@@ -30,6 +30,7 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
         Choose one of
 
         - ``Chebyshev`` or ``C``,
+        - ``Chebyshevu`` or ``U``,
         - ``Legendre`` or ``L``,
         - ``Fourier`` or ``F``,
         - ``Laguerre`` or ``La``,
@@ -81,27 +82,36 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
           - GL - Chebyshev-Gauss-Lobatto
           - GC - Chebyshev-Gauss
 
+        * For family=Chebyshevu:
+
+          - GU - Chebyshevu-Gauss
+          - GC - Chebyshev-Gauss
+
         * For family=Legendre:
 
           - LG - Legendre-Gauss
           - GL - Legendre-Gauss-Lobatto
+
         * For family=Laguerre:
 
           - LG - Laguerre-Gauss
+
         * For family=Hermite:
 
           - HG - Hermite-Gauss
+
         * For family=Jacobi:
 
           - JG - Jacobi-Gauss
+
     domain : two-tuple of floats, optional
         The computational domain
     scaled : bool
         Whether to use scaled basis (only Legendre)
     basis : str
         Name of basis to use, if there are more than one possible basis for a given
-        boundary condition. For example, there are two Dirichlet bases for the
-        Chebyshev family: 'Heinricht' and 'ShenDirichlet'
+        boundary condition. For example, there are three Dirichlet bases for the
+        Chebyshev family: 'Heinricht', 'ShenDirichlet' and 'Phi1'
     padding_factor : float, optional
         For padding backward transform (for dealiasing)
     dealias_direct : bool, optional
@@ -149,8 +159,8 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
                     bcs['right'] = [('D', bc[1])]
 
             if len(bc) == 4:
-                bcs['left'] = [('D', bc[0]), ('N', bc[2])]
-                bcs['right'] = [('D', bc[1]), ('N', bc[3])]
+                bcs['left'] = [('D', bc[0]), ('N', bc[1])]
+                bcs['right'] = [('D', bc[2]), ('N', bc[3])]
             key = ['L'+bci[0] for bci in bcs['left']] + ['R'+bci[0] for bci in bcs['right']]
 
         elif isinstance(bc, dict):
@@ -181,7 +191,6 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
                     else:
                         bc.append(bci[1])
                 key += ['R'+bci[0] for bci in bcs['right']]
-
         return key, tuple(bc)
 
     if family.lower() in ('fourier', 'f'):
@@ -215,6 +224,7 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             'RD': chebyshev.bases.UpperDirichlet,
             'LD': chebyshev.bases.LowerDirichlet,
             'RDRN': chebyshev.bases.UpperDirichletNeumann,
+            'LDLN': chebyshev.bases.LowerDirichletNeumann,
             'LDLNRDRN': chebyshev.bases.ShenBiharmonic
         }
 
@@ -278,6 +288,39 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
 
         return B(N, **par)
 
+    elif family.lower() in ('chebyshevu', 'u'):
+        from shenfun import chebyshevu
+        if quad is not None:
+            assert quad in ('GC', 'GU')
+            par['quad'] = quad
+
+        if scaled is not None:
+            assert isinstance(scaled, bool)
+            par['scaled'] = scaled
+
+        bases = {
+            '': chebyshevu.bases.Orthogonal,
+            'LDRD': chebyshevu.bases.CompactDirichlet,
+            'LDLNRDRN': chebyshevu.bases.Phi2
+        }
+
+        if isinstance(bc, (tuple, dict)):
+            key, par['bc'] = _process_bcs(bc, domain)
+
+        elif bc is None:
+            key = ''
+
+        else:
+            raise NotImplementedError
+
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(chebyshevu.bases, basis)
+        else:
+            B = bases[''.join(key)]
+
+        return B(N, **par)
+
     elif family.lower() in ('laguerre', 'la'):
         from shenfun import laguerre
         if quad is not None:
@@ -324,30 +367,37 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     elif family.lower() in ('jacobi', 'j'):
         from shenfun import jacobi
         if quad is not None:
-            assert quad in ('JG',)
+            assert quad in ('JG', )
             par['quad'] = quad
 
-        if bc is None:
-            B = jacobi.bases.Orthogonal
+        if scaled is not None:
+            assert isinstance(scaled, bool)
+            par['scaled'] = scaled
 
-        elif isinstance(bc, tuple):
-            assert len(bc) in (2, 4)
-            par['bc'] = bc
-            if len(bc) == 2:
-                B = jacobi.bases.ShenDirichlet
-            else:
-                assert np.all([abs(bci) < 1e-12 for bci in bc])
-                B = jacobi.bases.ShenBiharmonic
+        bases = {
+            '': jacobi.bases.Orthogonal,
+            'LDRD': jacobi.bases.CompactDirichlet,
+            'LDLNRDRN': jacobi.bases.Phi2,
+            '6th order': jacobi.bases.ShenOrder6
+        }
 
-        elif isinstance(bc, str):
-            if bc.lower() == 'dirichlet':
-                B = jacobi.bases.ShenDirichlet
-            elif bc.lower() == 'biharmonic':
-                B = jacobi.bases.ShenBiharmonic
-            elif bc.lower() == '6th order':
-                B = jacobi.bases.ShenOrder6
-            else:
-                raise NotImplementedError
+        if isinstance(bc, (tuple, dict)):
+            key, par['bc'] = _process_bcs(bc, domain)
+
+        elif bc is None:
+            key = ''
+
+        elif bc == '6th order':
+            key = bc
+
+        else:
+            raise NotImplementedError
+
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(jacobi.bases, basis)
+        else:
+            B = bases[''.join(key)]
 
         return B(N, **par)
 
@@ -576,8 +626,7 @@ class Expr:
                 if self.expr_rank() == 1:
                     s += "\\right) \\mathbf{%s}_{%s} \\\\"%(bl, symbols[x[i]])
                 elif self.expr_rank() == 2:
-                    ij = tuple([bl]+[symbols[xj] for xj in xx[i]])
-                    s += "\\right) \\mathbf{%s}_{%s} \\mathbf{b}_{%s} \\\\"%(ij)
+                    s += "\\right) \\mathbf{%s}_{%s} \\mathbf{%s}_{%s} \\\\"%(bl, symbols[xx[i][0]], bl, symbols[xx[i][1]])
                 s += '+'
 
         return r"""%s"""%(s.rstrip('+'))
@@ -743,8 +792,10 @@ class Expr:
             raise IndexError
 
         basis = self._basis
-        if self.expr_rank() == self.tensor_rank and self.is_basis_function:
-            basis = self._basis[i]
+        if self.tensor_rank:
+            if ((self.expr_rank() == self.tensor_rank and self.is_basis_function) or
+                (self.expr_rank() > self.tensor_rank and self.tensor_rank > 0)):
+                basis = self._basis[i]
 
         if self.expr_rank() == 1:
             return Expr(basis,
@@ -900,7 +951,11 @@ class Expr:
         return self
 
     def __neg__(self):
-        return Expr(self.basis(), copy.deepcopy(self.terms()), (-np.array(self.scales())).tolist(),
+        sc = copy.deepcopy(self.scales())
+        for vec in sc:
+            for j in range(len(vec)):
+                vec[j] = vec[j]*(-1)
+        return Expr(self.basis(), copy.deepcopy(self.terms()), sc,
                     copy.deepcopy(self.indices()))
 
     def simplify(self):
@@ -1177,6 +1232,8 @@ class ShenfunBaseArray(DistArray):
                                     val=val0, rank=space.is_composite_space)
             obj._space = space
             obj._offset = 0
+            if cls.__name__ == 'Function':
+                obj._padded_space = {}
             if buffer is None and isinstance(val, (list, tuple)):
                 assert len(val) == len(obj)
                 for i, v in enumerate(val):
@@ -1251,6 +1308,9 @@ class ShenfunBaseArray(DistArray):
         if isinstance(val, (list, tuple)):
             for i, v in enumerate(val):
                 obj.v[i] = v
+        if cls.__name__ == 'Function':
+            obj._padded_space = {}
+
         return obj
 
     def __array_finalize__(self, obj):
@@ -1260,6 +1320,8 @@ class ShenfunBaseArray(DistArray):
         self._p0 = getattr(obj, '_p0', None)
         self._rank = getattr(obj, '_rank', None)
         self._offset = getattr(obj, '_offset', None)
+        if hasattr(obj, '_padded_space'):
+            self._padded_space = obj._padded_space
 
     def function_space(self):
         """Return function space of array ``self``"""
@@ -1333,10 +1395,9 @@ class ShenfunBaseArray(DistArray):
 
 class Function(ShenfunBaseArray, BasisFunction):
     r"""
-    Spectral Galerkin function for given :class:`.TensorProductSpace` or :func:`.Basis`
+    Spectral Galerkin function for given :class:`.TensorProductSpace` or :func:`.SpectralBase`
 
-    The Function is the product of all 1D basis expansions, that for each
-    dimension is defined like
+    The Function in one dimension is
 
     .. math::
 
@@ -1378,7 +1439,12 @@ class Function(ShenfunBaseArray, BasisFunction):
         If array it must be of correct shape.
         A sympy expression is evaluated on the quadrature mesh and
         forward transformed to create the buffer array.
-
+    reltol : number, optional
+        Relative tolerance for adaptively finding dimension of space from
+        the buffer.
+    abstol : number, optional
+        Absolute tolerance for adaptively finding dimension of space from
+        the buffer
 
     .. note:: For more information, see `numpy.ndarray <https://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html>`_
 
@@ -1447,13 +1513,11 @@ class Function(ShenfunBaseArray, BasisFunction):
         return self.eval(x, output_array=output_array)
 
     def eval(self, x, output_array=None):
-        """Evaluate Function at points
+        """Evaluate Function at points `x`
 
         Parameters
         ----------
-        points : float or array of floats
-        coefficients : array
-            Expansion coefficients
+        x : float or array of floats
         output_array : array, optional
             Return array, function values at points
 
@@ -1475,7 +1539,7 @@ class Function(ShenfunBaseArray, BasisFunction):
         """
         return self.function_space().eval(x, self, output_array)
 
-    def backward(self, output_array=None, kind='normal'):
+    def backward(self, output_array=None, kind='normal', padding_factor=None):
         """Return Function evaluated on some quadrature mesh
 
         Parameters
@@ -1490,8 +1554,29 @@ class Function(ShenfunBaseArray, BasisFunction):
             - function space - evaluate Function on the quadrature mesh of the
               given function space. This could be a :func:`.FunctionSpace`
               if 1D, or a :class:`.TensorProductSpace` if multidimensional.
+        padding_factor : None or number, optional
+            If padding_factor is a number different from 1, then perform a
+            padded backward transform, using a padded work array
+            'self._backward_work_array'
 
         """
+        if padding_factor is not None:
+            space = self.get_dealiased_space(padding_factor)
+            if output_array is not None:
+                assert output_array.shape == space.shape(False)
+            else:
+                # Create a work array that can be reused
+                # Note that the output_array is not the same shape as
+                # space.backward.output_array for spaces of rank > 0.
+                # Also, it is probably not safe to return space.backward.output_array
+                if hasattr(space, '_backward_work_array'):
+                    output_array = space._backward_work_array
+                else:
+                    output_array = Array(space)
+                    space._backward_work_array = output_array
+            output_array = space.backward(self, output_array)
+            return output_array
+
         space = self.function_space()
         if hasattr(kind, 'mesh'):
             if output_array is None:
@@ -1507,6 +1592,24 @@ class Function(ShenfunBaseArray, BasisFunction):
             output_array = space.backward(self, output_array)
         return output_array
 
+    def get_dealiased_space(self, padding_factor):
+        """Return a function space to be used for padded transforms
+
+        Parameters
+        ----------
+        padding_factor : number
+            The padding factor of the backward transform
+        """
+        if padding_factor is None:
+            return self.function_space()
+        if abs(padding_factor-1) < 1e-8:
+            return self.function_space()
+        if padding_factor in self._padded_space:
+            return self._padded_space[padding_factor]
+        space = self.function_space().get_dealiased(padding_factor)
+        self._padded_space[padding_factor] = space
+        return space
+
     def to_ortho(self, output_array=None):
         """Project Function to orthogonal basis"""
         space = self.function_space()
@@ -1679,6 +1782,67 @@ class Function(ShenfunBaseArray, BasisFunction):
             output_array[:] = c1
         return output_array
 
+    def copy_to_flattened(self, f=None, j=(), dims=None, sl=None):
+        """Copy dofs of self to a flattened array
+
+        Parameters
+        ----------
+        f : array 1D, optional
+            The array to copy to
+        j : int or tuple, optional
+            Index into diagonal axes (Fourier). Not used if sl is provided
+        dims : array 1D, optional
+            The dim of each variable in the composite basis
+        sl : sequence of slices, optional
+            list of slices into non-diagonal axes
+
+        Note
+        ----
+        All Functions have allocated Numpy arrays assuming all dofs are being
+        used. However, for non-orthogonal bases with boundary conditions we use
+        only the first N-nb dofs, where nb is the number of boundary conditions.
+        This function does not copy the boundary dofs.
+        """
+        if dims is None:
+            dims = self.function_space().get_ndiag_cum_dofs()
+        if sl is None:
+            sl = self.function_space().get_ndiag_slices(j)
+        if f is None:
+            f = np.zeros(dims[-1], dtype=self.dtype)
+        for i, s in enumerate(sl):
+            f[dims[i]:dims[i+1]] = self[tuple(s)].ravel()
+        return f
+
+    def copy_from_flattened(self, f, j=(), dims=None, sl=None):
+        """Copy dofs to self from a flattened array
+
+        Parameters
+        ----------
+        f : array 1D
+            The array to copy from
+        j : int or tuple, optional
+            Index into diagonal axes (Fourier)
+        dims : array 1D, optional
+            The flat dim of each variable in the composite basis
+        sl : sequence of slices, optional
+            list of slices into non-diagonal axes
+
+        Note
+        ----
+        All Functions have allocated Numpy arrays assuming all dofs are being
+        used. However, for non-orthogonal bases with boundary conditions we use
+        only the first N-nb dofs, where nb is the number of boundary conditions.
+        This function does not copy the boundary dofs.
+        """
+        if dims is None:
+            dims = self.function_space().get_ndiag_cum_dofs()
+        if sl is None:
+            sl = self.function_space().get_ndiag_slices(j)
+        for i, s in enumerate(sl):
+            si = tuple(s)
+            self[si] = f[dims[i]:dims[i+1]].reshape(self[si].shape)
+        return self
+
 
 class Array(ShenfunBaseArray):
     r"""
@@ -1687,8 +1851,7 @@ class Array(ShenfunBaseArray):
     The Array is the result of a :class:`.Function` evaluated on its quadrature
     mesh.
 
-    The Function is the product of all 1D basis expansions, that for each
-    dimension is defined like
+    The Function represents in 1D
 
     .. math::
 
@@ -1772,3 +1935,27 @@ class Array(ShenfunBaseArray):
         """
         return self._offset
 
+    def get_cartesian_vector(self):
+        """Return self as a Cartesian vector.
+
+        Note
+        ----
+        This method is only relevant for curvilinear coordinates, where
+        the vector uses different basis functions from the Cartesian.
+        """
+        assert self.tensor_rank == 1
+        T = self.function_space()
+        psi = T.coors.psi
+        ndim = len(psi)
+        x = T.local_mesh(True)
+        b = T.coors.get_basis()
+        bij = np.array(sp.lambdify(psi, b)(*x), dtype=object)
+        for bi in np.ndindex(bij.shape):
+            if isinstance(bij[bi], int):
+                bij[bi] = np.full(x[0].shape, bij[bi])
+        bij = np.array(bij.tolist())
+        sl = [slice(None)]*bij.ndim
+        sl[1] = None
+        uc = np.sum(self[tuple(sl)]*bij, axis=0)
+        return uc
+
