@@ -111,7 +111,8 @@ class FourierBase(SpectralBase):
         if map_true_domain is True:
             points = self.map_true_domain(points)
         if weighted:
-            return points, np.array([1/N])
+            # weight is 1/self.domain_length() since this leads to the Kronecker delta function for mass matrix
+            return points, np.array([self.domain_factor()/N])
         return points, np.array([2*np.pi/N])
 
     def sympy_basis(self, i=0, x=sp.symbols('x', real=True)):
@@ -119,7 +120,7 @@ class FourierBase(SpectralBase):
         return sp.exp(sp.I*k[i]*x)
 
     def weight(self, x=sp.symbols('x', real=True)):
-        return 1/(2*sp.pi)
+        return 1/self.domain_length()
 
     def evaluate_basis(self, x=None, i=0, output_array=None):
         if x is None:
