@@ -9,7 +9,7 @@ from collections.abc import Mapping, MutableMapping
 from numbers import Number, Integral
 import numpy as np
 import sympy as sp
-from scipy.sparse import bmat, dia_matrix, kron, diags as sp_diags
+from scipy.sparse import bmat, dia_matrix, csr_matrix, kron, diags as sp_diags
 from scipy.sparse.linalg import spsolve
 from mpi4py import MPI
 from shenfun.config import config
@@ -19,7 +19,7 @@ __all__ = ['SparseMatrix', 'SpectralMatrix', 'extract_diagonal_matrix',
            'extract_bc_matrices', 'check_sanity', 'get_dense_matrix',
            'TPMatrix', 'BlockMatrix', 'BlockMatrices', 'Identity',
            'get_dense_matrix_sympy', 'get_dense_matrix_quadpy',
-           'get_simplified_tpmatrices']
+           'get_simplified_tpmatrices', 'ScipyMatrix']
 
 comm = MPI.COMM_WORLD
 
@@ -198,9 +198,6 @@ class SparseMatrix(MutableMapping):
     def __len__(self):
         return len(self._storage)
 
-    def __quasi__(self, Q):
-        return Q.diags('csc')*self.diags('csc')
-
     def __eq__(self, a):
         if self.shape != a.shape:
             return False
@@ -447,7 +444,7 @@ class SparseMatrix(MutableMapping):
         Fall back on generic Solve, which is using Scipy sparse
         matrices with splu/spsolve. This is still pretty fast.
         """
-        from .la import Solve, TDMA, TDMA_O, FDMA, TwoDMA, PDMA, DiagMA
+        from .la import Solve, TDMA, TDMA_O, FDMA, TwoDMA, ThreeDMA, PDMA, DiagMA
         if len(self) == 1:
             if list(self.keys())[0] == 0:
                 return DiagMA
@@ -459,6 +456,8 @@ class SparseMatrix(MutableMapping):
                 return TDMA
             elif np.all(self.sorted_keys() == (-1, 0, 1)) and self.issymmetric:
                 return TDMA_O
+            elif np.all(self.sorted_keys() == (0, 2, 4)):
+                return ThreeDMA
         elif len(self) == 4:
             if np.all(self.sorted_keys() == (-2, 0, 2, 4)):
                 return FDMA
@@ -533,6 +532,7 @@ class SpectralMatrix(SparseMatrix):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -591,9 +591,17 @@ class SpectralMatrix(SparseMatrix):
                 D = get_dense_matrix(test, trial, measure)[:shape[0], :shape[1]]
             elif config['matrix']['sparse']['construct'] == 'denser':
                 D = get_denser_matrix(test, trial, measure)[:shape[0], :shape[1]]
-            else:
+            elif config['matrix']['sparse']['construct'] == 'sympy':
                 D = get_dense_matrix_sympy(test, trial, measure)[:shape[0], :shape[1]]
+            elif config['matrix']['sparse']['construct'] == 'quadpy':
+                D = get_dense_matrix_quadpy(test, trial, measure)[:shape[0], :shape[1]]
             d = extract_diagonal_matrix(D)
+        else:
+            if isinstance(measure, Number):
+                scale *= measure
+                self.measure = 1
+        if test[0].domain_factor() != 1:
+            scale *= test[0].domain_factor()**(test[1]+trial[1]-1)
         SparseMatrix.__init__(self, d, shape, scale)
 
     def matvec(self, v, c, format=None, axis=0):
@@ -674,6 +682,57 @@ class Identity(SparseMatrix):
         u *= (1/self.scale)
         return u
 
+class ScipyMatrix(csr_matrix):
+
+    def __init__(self, mats):
+        assert isinstance(mats, (SparseMatrix, list))
+        self.bc_mats = []
+        if isinstance(mats, list):
+            bc_mats = extract_bc_matrices([mats])
+            mats = sum(mats[1:], mats[0])
+            self.bc_mats = bc_mats
+        csr_matrix.__init__(self, mats.diags('csr'))
+
+    def matvec(self, v, c, axis=0):
+        """Matrix vector product
+
+        Returns c = dot(self, v)
+
+        Parameters
+        ----------
+        v : array
+            Numpy input array of ndim>=1
+        c : array
+            Numpy output array of same shape as v
+        axis : int, optional
+            The axis over which to take the matrix vector product
+
+        """
+        N, M = self.shape
+        c.fill(0)
+
+        # Roll relevant axis to first
+        if axis > 0:
+            v = np.moveaxis(v, axis, 0)
+            c = np.moveaxis(c, axis, 0)
+
+        P = int(np.prod(v.shape[1:]))
+        y = self.dot(v[:M].reshape(M, P)).squeeze()
+        d = tuple([slice(0, m) for m in y.shape])
+        c[d] = y.reshape(c[d].shape)
+
+        if self.bc_mats:
+            w0 = np.zeros_like(c)
+            for bc_mat in self.bc_mats:
+                c += bc_mat.matvec(v, w0, axis=0)
+
+        if axis > 0:
+            c = np.moveaxis(c, 0, axis)
+            v = np.moveaxis(v, 0, axis)
+
+        return c
+
+
 def BlockMatrices(tpmats):
     """Return two instances of the :class:`.BlockMatrix` class.
 
@@ -779,6 +838,7 @@ class BlockMatrix:
         self.trialbase = trialbase = tpmats[0][0].trialbase
         self.dims = dims = (testbase.num_components(), trialbase.num_components())
         self.mats = np.zeros(dims, dtype=int).tolist()
+        self._Ai = None
         self.solver = None
         self += tpmats
 
@@ -831,7 +891,7 @@ class BlockMatrix:
                                 tpmats.append(m)
         return tpmats
 
-    def matvec(self, v, c, format=None):
+    def matvec(self, v, c, format=None, use_scipy=None):
         """Compute matrix vector product
 
             c = self * v
@@ -840,7 +900,14 @@ class BlockMatrix:
         ----------
         v : :class:`.Function`
         c : :class:`.Function`
+        format : str, optional
+            The format of the matrices used for the matvec.
+            See `Scipy sparse matrices <https://docs.scipy.org/doc/scipy/reference/sparse.html>`_
 
+        use_scipy : boolean, optional
+            Whether to assemble and use scipy's bmat for the matvec, or to use
+            the matvec methods of this BlockMatrix's TPMatrices.
+            Using config['matrix']['block']['use_scipy'] if use_scipy is None
         Returns
         -------
         c : :class:`.Function`
@@ -849,30 +916,50 @@ class BlockMatrix:
         assert v.function_space() == self.trialbase
         assert c.function_space() == self.testbase
         nvars = c.function_space().num_components()
-        c = c.reshape(1, *c.shape) if nvars == 1 else c
-        v = v.reshape(1, *v.shape) if nvars == 1 else v
+        c = np.expand_dims(c, 0) if nvars == 1 else c
+        v = np.expand_dims(v, 0) if nvars == 1 else v
         c.v.fill(0)
-        z = np.zeros_like(c.v[0])
-        for i, mi in enumerate(self.mats):
-            for j, mij in enumerate(mi):
-                if isinstance(mij, Number):
-                    if abs(mij) > 1e-8:
-                        c.v[i] += mij*v.v[j]
-                else:
-                    for m in mij:
-                        z.fill(0)
-                        z = m.matvec(v.v[j], z, format=format)
-                        c.v[i] += z
-        c = c.squeeze() if nvars == 1 else c
-        v = v.squeeze() if nvars == 1 else v
+        use_scipy = config['matrix']['block']['use_scipy'] if use_scipy is None else use_scipy
+        if self.contains_bc_matrix():
+            use_scipy = False
+
+        if use_scipy:
+            self.assemble(format)
+            daxes = self.testbase.get_diagonal_axes()
+            if len(daxes) > 0:
+                daxes += 1
+            sl1, dims1 = self.trialbase.get_ndiag_slices_and_dims()
+            sl2, dims2 = self.testbase.get_ndiag_slices_and_dims()
+            gi = np.zeros(dims1[-1], dtype=v.dtype)
+            go = np.zeros(dims2[-1], dtype=v.dtype)
+            for key, val in self._Ai.items():
+                key = np.atleast_1d(key)
+                if len(daxes) > 0:
+                    sl1.T[daxes] = np.array(key)[:, None]
+                    sl2.T[daxes] = np.array(key)[:, None]
+                gi = v.copy_to_flattened(gi, key, dims1, sl1)
+                go[:] = val * gi
+                c = c.copy_from_flattened(go, key, dims2, sl2)
+
+        else:
+            z = np.zeros_like(c.v[0])
+            for i, mi in enumerate(self.mats):
+                for j, mij in enumerate(mi):
+                    if isinstance(mij, Number):
+                        if abs(mij) > 1e-8:
+                            c.v[i] += mij*v.v[j]
+                    else:
+                        for m in mij:
+                            z.fill(0)
+                            z = m.matvec(v.v[j], z, format=format)
+                            c.v[i] += z
+        c = c.reshape(c.shape[1:]) if nvars == 1 else c
+        v = v.reshape(v.shape[1:]) if nvars == 1 else v
         return c
 
     def __getitem__(self, ij):
         return self.mats[ij[0]][ij[1]]
 
-    def get_offset(self, i, axis=0):
-        return self.offset[i][axis]
-
     def contains_bc_matrix(self):
         """Return True if self contains a boundary TPMatrix"""
         for mi in self.mats:
@@ -892,7 +979,38 @@ class BlockMatrix:
                             return True
         return False
 
-    def diags(self, it=(0,), format=None):
+    def assemble(self, format=None):
+        """Assemble matrices in scipy sparse format
+
+        Parameters
+        ----------
+        format : str or None, optional
+            The format of the sparse scipy matrix. Using
+            config['matrix']['block']['assemble'] if None.
+        """
+        if self._Ai is not None:
+            return
+        self._Ai = {}
+        tpmat = self.get_mats(True)
+        N = self.testbase.forward.output_array.shape
+        daxes = self.get_diagonal_axes()
+        ndindices = [(0,)] if len(daxes) == 0 else np.ndindex(tuple(np.array(N)[daxes]))
+        format = config['matrix']['block']['assemble'] if format is None else format
+        for i in ndindices:
+            i = i[0] if len(i) == 1 else i
+            if format == 'csc':
+                Ai = self.diags(i, format='csr').tocsc() # because of bug in scipy
+            else:
+                Ai = self.diags(i, format=format)
+            self._Ai[i] = Ai
+
+    def get_diagonal_axes(self):
+        if self.testbase.dimensions == 1:
+            return np.array([])
+        tpmat = self.get_mats(True)
+        return np.setxor1d(tpmat.naxes, range(tpmat.dimensions)).astype(int)
+
+    def diags(self, it=None, format=None):
         """Return global block matrix in scipy sparse format
 
         For multidimensional forms the returned matrix is constructed for
@@ -900,12 +1018,12 @@ class BlockMatrix:
 
         Parameters
         ----------
-        it : n-tuple of ints
-            where n is dimensions. These are the indices into the scale arrays
-            of the TPMatrices in various blocks. Should be zero along the non-
-            periodic direction.
-        format : str
+        it : n-tuple of ints or None, optional
+            where n is dimensions-1. These are the indices into the diagonal
+            axes, or the axes with Fourier bases.
+        format : str or None, optional
             The format of the returned matrix. See `Scipy sparse matrices <https://docs.scipy.org/doc/scipy/reference/sparse.html>`_
+            If None, then use default for TPMatrix.
 
         """
         from .spectralbase import MixedFunctionSpace
@@ -919,30 +1037,31 @@ class BlockMatrix:
                     bm[-1].append(None)
                 else:
                     m = mij[0]
-                    if isinstance(self.testbase, MixedFunctionSpace):
+                    if isinstance(self.testbase, MixedFunctionSpace) or len(m.naxes) == len(m.mats):
                         d = m.diags(format)
                         for mj in mij[1:]:
                             d = d + mj.diags(format)
-                    elif len(m.naxes) == 2: # 2 non-periodic directions
-                        if len(m.mats) == 2:
-                            d = m.scale.item()*kron(m.mats[0].diags(format), m.mats[1].diags(format))
-                            for mj in mij[1:]:
-                                d = d + mj.scale.item()*kron(mj.mats[0].diags(format), mj.mats[1].diags(format))
-                        else:
-                            iit = np.where(np.array(m.scale.shape) == 1, 0, it) # if shape is 1 use index 0, else use given index (shape=1 means the scale is constant in that direction)
-                            d = m.scale[tuple(iit)]*kron(m.mats[m.naxes[0]].diags(format=format), m.mats[m.naxes[1]].diags(format=format))
-                            for mj in mij[1:]:
-                                iit = np.where(np.array(mj.scale.shape) == 1, 0, it)
-                                sc = mj.scale[tuple(iit)]
-                                d = d + sc*kron(mj.mats[mj.naxes[0]].diags(format=format), mj.mats[mj.naxes[1]].diags(format=format))
 
-                    else:
+                    elif len(m.naxes) == 2: # 2 non-periodic directions
                         iit = np.where(np.array(m.scale.shape) == 1, 0, it) # if shape is 1 use index 0, else use given index (shape=1 means the scale is constant in that direction)
-                        sc = m.scale[tuple(iit)]
-                        d = sc*m.mats[m.naxes[0]].diags(format)
+                        d = m.scale[tuple(iit)]*kron(m.mats[m.naxes[0]].diags(format=format), m.mats[m.naxes[1]].diags(format=format))
                         for mj in mij[1:]:
                             iit = np.where(np.array(mj.scale.shape) == 1, 0, it)
                             sc = mj.scale[tuple(iit)]
+                            d = d + sc*kron(mj.mats[mj.naxes[0]].diags(format=format), mj.mats[mj.naxes[1]].diags(format=format))
+
+                    else:
+                        assert len(m.naxes) == 1
+                        iit = np.zeros(m.dimensions, dtype=int)
+                        diagonal_axes = self.get_diagonal_axes()
+                        assert len(diagonal_axes) + len(m.naxes) == m.dimensions
+                        iit[diagonal_axes] = it
+                        ij = np.where(np.array(m.scale.shape) == 1, 0, iit) # if shape is 1 use index 0, else use given index (shape=1 means the scale is constant in that direction)
+                        sc = m.scale[tuple(ij)]
+                        d = sc*m.mats[m.naxes[0]].diags(format)
+                        for mj in mij[1:]:
+                            ij = np.where(np.array(mj.scale.shape) == 1, 0, iit)
+                            sc = mj.scale[tuple(ij)]
                             d = d + sc*mj.mats[mj.naxes[0]].diags(format)
                     bm[-1].append(d)
         return bmat(bm, format=format)
@@ -1168,7 +1287,7 @@ class TPMatrix:
             c[:] = c*tpmat.scale
         elif len(tpmat.naxes) == 2:
             # 2 non-periodic directions (may be non-aligned in second axis, hence transfers)
-            npaxes = deepcopy(tpmat.naxes)
+            npaxes = deepcopy(list(tpmat.naxes))
             space = tpmat.space
             newspace = False
             if space.forward.input_array.shape != space.forward.output_array.shape:
@@ -1370,7 +1489,7 @@ class TPMatrix:
 
         return mat*np.atleast_1d(self.scale).item()
 
-def get_simplified_tpmatrices(tpmats : List[TPMatrix]) -> List[TPMatrix]:
+def get_simplified_tpmatrices(tpmats):
     """Return copy of tpmats list, where diagonal matrices have been
     simplified and placed in scale arrays.
 
@@ -1415,6 +1534,7 @@ def check_sanity(A, test, trial, measure=1):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1445,6 +1565,7 @@ def get_dense_matrix(test, trial, measure=1):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1457,6 +1578,23 @@ def get_dense_matrix(test, trial, measure=1):
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
+
+    Note
+    ----
+    The computed matrix is not compensated for a non-standard domain size.
+    This is because all pre-computed matrices use the reference domain, and
+    compensate for the domain size later. This function is a drop-in for all
+    pre-computed matrices, and thus needs to assume a standard reference domain.
+    To create a true matrix with this function, do not use it directly, but
+    wrapped in the SpectralMatrix class, like
+
+    >>> from shenfun import config, SpectralMatrix, FunctionSpace
+    >>> config['matrix']['sparse']['construct'] = 'dense'
+    >>> L = FunctionSpace(8, 'L', domain=(-2, 2))
+    >>> D = SpectralMatrix({}, (L, 0), (L, 0))
+    >>> D.scale
+    2.0
+
     """
     K0 = test[0].slice().stop - test[0].slice().start
     K1 = trial[0].slice().stop - trial[0].slice().start
@@ -1486,6 +1624,7 @@ def get_denser_matrix(test, trial, measure=1):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1498,9 +1637,25 @@ def get_denser_matrix(test, trial, measure=1):
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
+
+    Note
+    ----
+    The computed matrix is not compensated for a non-standard domain size.
+    This is because all pre-computed matrices use the reference domain, and
+    compensate for the domain size later. This function is a drop-in for all
+    pre-computed matrices, and thus needs to assume a standard reference domain.
+    To create a true matrix with this function, do not use it directly, but
+    wrapped in the SpectralMatrix class, like
+
+    >>> from shenfun import config, SpectralMatrix, FunctionSpace
+    >>> config['matrix']['sparse']['construct'] = 'denser'
+    >>> L = FunctionSpace(8, 'L', domain=(-2, 2))
+    >>> D = SpectralMatrix({}, (L, 0), (L, 0))
+    >>> D.scale
+    2.0
+
     """
     test2 = test[0].get_refined((test[0].N*3)//2)
-
     K0 = test[0].slice().stop - test[0].slice().start
     K1 = trial[0].slice().stop - trial[0].slice().start
     N = test2.N
@@ -1508,14 +1663,25 @@ def get_denser_matrix(test, trial, measure=1):
     ws = test2.get_measured_weights(N, measure)
     v = test[0].evaluate_basis_derivative_all(x=x, k=test[1])[:, :K0]
     u = trial[0].evaluate_basis_derivative_all(x=x, k=trial[1])[:, :K1]
-    return np.dot(np.conj(v.T)*ws[np.newaxis, :], u)
+    A = np.dot(np.conj(v.T)*ws[np.newaxis, :], u)
+    if A.dtype.char in 'FDG':
+        ni = np.linalg.norm(A.imag)
+        if ni == 0:
+            A = A.real.copy()
+        elif np.linalg.norm(A.real) / ni > 1e14:
+            A = A.real.copy()
+    return A
 
-def extract_diagonal_matrix(M, abstol=1e-10, reltol=1e-10):
+def extract_diagonal_matrix(M, lowerband=None, upperband=None, abstol=1e-10, reltol=1e-10):
     """Return SparseMatrix version of dense matrix ``M``
 
     Parameters
     ----------
     M : Numpy array of ndim=2
+    lowerband : int or None
+        Assumed lower bandwidth of M
+    upperband : int or None
+        Assumed upper bandwidth of M
     abstol : float
         Tolerance. Only diagonals with max(:math:`|d|`) < tol are
         kept in the returned SparseMatrix, where :math:`d` is the
@@ -1529,16 +1695,12 @@ def extract_diagonal_matrix(M, abstol=1e-10, reltol=1e-10):
     d = {}
     relmax = abs(M).max()
     dtype = float if M.dtype == 'O' else M.dtype # For mpf object
-    for i in range(M.shape[1]):
+    upperband = M.shape[1] if upperband is None else min(upperband+1, M.shape[1])
+    lowerband = M.shape[0]-1 if lowerband is None else min(lowerband, M.shape[0]-1)
+    for i in range(-lowerband, upperband):
         u = M.diagonal(i).copy()
         if abs(u).max() > abstol and abs(u).max()/relmax > reltol:
             d[i] = np.array(u, dtype=dtype)
-
-    for i in range(1, M.shape[0]):
-        l = M.diagonal(-i).copy()
-        if abs(l).max() > abstol and abs(l).max()/relmax > reltol:
-            d[-i] = np.array(l, dtype=dtype)
-
     return SparseMatrix(d, M.shape)
 
 def extract_bc_matrices(mats):
@@ -1577,6 +1739,7 @@ def get_dense_matrix_sympy(test, trial, measure=1):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1589,6 +1752,24 @@ def get_dense_matrix_sympy(test, trial, measure=1):
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
+
+    Note
+    ----
+    The computed matrix is not compensated for a non-standard domain size.
+    This is because all pre-computed matrices use the reference domain, and
+    compensate for the domain size later. This function is a drop-in for all
+    pre-computed matrices, and thus needs to assume a standard reference domain.
+    To create a true matrix with this function, do not use it directly, but
+    wrapped in the SpectralMatrix class, like
+
+    >>> from shenfun import config, SpectralMatrix, FunctionSpace
+    >>> config['matrix']['sparse']['construct'] = 'sympy'
+    >>> L = FunctionSpace(8, 'L', domain=(-2, 2))
+    >>> D = SpectralMatrix({}, (L, 0), (L, 0))
+    >>> D.scale
+    2.0
+    >>> config['matrix']['sparse']['construct'] = 'dense' # reset for doctest
+
     """
     N = test[0].slice().stop - test[0].slice().start
     M = trial[0].slice().stop - trial[0].slice().start
@@ -1605,17 +1786,22 @@ def get_dense_matrix_sympy(test, trial, measure=1):
         else:
             assert isinstance(measure, Number)
 
-    # Weight of weighted space
-    measure *= test[0].weight()
+    measure *= test[0].weight() # Weight of weighted space (in reference domain)
 
     for i in range(test[0].slice().start, test[0].slice().stop):
         pi = np.conj(test[0].sympy_basis(i, x=x))
         for j in range(trial[0].slice().start, trial[0].slice().stop):
             pj = trial[0].sympy_basis(j, x=x)
-            integrand = sp.simplify(measure*pi.diff(x, test[1])*pj.diff(x, trial[1]))
+            integrand = measure*pi.diff(x, test[1])*pj.diff(x, trial[1])
             V[i, j] = integrate_sympy(integrand,
                                       (x, test[0].sympy_reference_domain()[0], test[0].sympy_reference_domain()[1]))
 
+    if V.dtype.char in 'FDG':
+        ni = np.linalg.norm(V.imag)
+        if ni == 0:
+            V = V.real.copy()
+        elif np.linalg.norm(V.real) / ni > 1e14:
+            V = V.real.copy()
     return V
 
 def get_dense_matrix_quadpy(test, trial, measure=1):
@@ -1633,6 +1819,7 @@ def get_dense_matrix_quadpy(test, trial, measure=1):
 
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
+        - :mod:`.chebyshevu.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1645,6 +1832,23 @@ def get_dense_matrix_quadpy(test, trial, measure=1):
     measure : Sympy expression of coordinate, or number, optional
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
+
+    Note
+    ----
+    The computed matrix is not compensated for a non-standard domain size.
+    This is because all pre-computed matrices use the reference domain, and
+    compensate for the domain size later. This function is a drop-in for all
+    pre-computed matrices, and thus needs to assume a standard reference domain.
+    To create a true matrix with this function, do not use it directly, but
+    wrapped in the SpectralMatrix class, like
+
+    >>> from shenfun import config, SpectralMatrix, FunctionSpace
+    >>> config['matrix']['sparse']['construct'] = 'quadpy'
+    >>> L = FunctionSpace(8, 'L', domain=(-2, 2))
+    >>> D = SpectralMatrix({}, (L, 0), (L, 0))
+    >>> D.scale
+    2.0
+
     """
     import quadpy
     N = test[0].slice().stop - test[0].slice().start
@@ -1664,14 +1868,22 @@ def get_dense_matrix_quadpy(test, trial, measure=1):
 
     # Weight of weighted space
     measure *= test[0].weight()
+    rd = test[0].reference_domain()
 
     for i in range(test[0].slice().start, test[0].slice().stop):
         pi = np.conj(test[0].sympy_basis(i, x=x))
         for j in range(trial[0].slice().start, trial[0].slice().stop):
             pj = trial[0].sympy_basis(j, x=x)
-            integrand = sp.simplify(measure*pi.diff(x, test[1])*pj.diff(x, trial[1]))
-            if not integrand == 0:
-                V[i, j] = quadpy.c1.integrate_adaptive(sp.lambdify(x, integrand),
-                                                       test[0].reference_domain())[0]
+            integrand = measure*pi.diff(x, test[1])*pj.diff(x, trial[1])
+            if isinstance(integrand, Number):
+                V[i, j] = integrand*(rd[1]-rd[0])
+            else:
+                V[i, j] = quadpy.c1.integrate_adaptive(sp.lambdify(x, integrand), rd)[0]
 
+    if V.dtype.char in 'FDG':
+        ni = np.linalg.norm(V.imag)
+        if ni == 0:
+            V = V.real.copy()
+        elif np.linalg.norm(V.real) / ni > 1e14:
+            V = V.real.copy()
     return V
