@@ -1,10 +1,10 @@
 from .la import TDMA_LU, TDMA_Solve, PDMA_LU, PDMA_Solve, \
     LU_Helmholtz, Solve_Helmholtz, LU_Biharmonic, Biharmonic_factor_pr, \
     Biharmonic_Solve, TDMA_O_Solve, TDMA_O_LU, Poisson_Solve_ADD, \
-    FDMA_Solve, TwoDMA_Solve, FDMA_LU, DiagMA_Solve, \
+    FDMA_Solve, TwoDMA_Solve, ThreeDMA_Solve, FDMA_LU, DiagMA_Solve, \
     TDMA_inner_solve, TDMA_O_inner_solve, DiagMA_inner_solve, \
     PDMA_inner_solve, FDMA_inner_solve, TwoDMA_inner_solve, \
-    SolverGeneric1ND_solve_data
+    ThreeDMA_inner_solve, SolverGeneric1ND_solve_data
 
 from .Matvec import Helmholtz_matvec, Helmholtz_Neumann_matvec, Biharmonic_matvec
 from .outer import outer2D, outer3D
