@@ -1,6 +1,7 @@
 import numba as nb
 
-__all__ = ['SolverGeneric1ND_solve_data']
+__all__ = ['SolverGeneric1ND_solve_data',
+           'Solve_axis_2D', 'Solve_axis_3D']
 
 @nb.njit
 def SolverGeneric1ND_solve_data(u, data, sol, naxes, is_zero_index):
@@ -38,4 +39,28 @@ def SolverGeneric1ND_solve_data(u, data, sol, naxes, is_zero_index):
                     if i == 0 and j == 0 and is_zero_index:
                         continue
                     sol(u[i, j, :], data[i, j])
-    return u
\ No newline at end of file
+    return u
+
+@nb.jit(nopython=True, fastmath=True, cache=True)
+def Solve_axis_2D(data, x, innerfun, axis):
+    if axis == 0:
+        for j in range(x.shape[1]):
+            innerfun(x[:, j], data)
+    elif axis == 1:
+        for i in range(x.shape[0]):
+            innerfun(x[i, :], data)
+
+@nb.jit(nopython=True, fastmath=True, cache=True)
+def Solve_axis_3D(data, x, innerfun, axis):
+    if axis == 0:
+        for j in range(x.shape[1]):
+            for k in range(x.shape[2]):
+                innerfun(x[:, j, k], data)
+    elif axis == 1:
+        for i in range(x.shape[0]):
+            for k in range(x.shape[2]):
+                innerfun(x[i, :, k], data)
+    elif axis == 2:
+        for i in range(x.shape[0]):
+            for j in range(x.shape[1]):
+                innerfun(x[i, j], data)
