@@ -192,7 +192,7 @@ class SpectralBase:
         Example
         -------
         >>> import numpy as np
-        >>> from shenfun import Basis, TensorProductSpace, comm
+        >>> from shenfun import FunctionSpace, TensorProductSpace, comm
         >>> K0 = FunctionSpace(8, 'F', dtype='D')
         >>> K1 = FunctionSpace(8, 'F', dtype='d')
         >>> T = TensorProductSpace(comm, (K0, K1), modify_spaces_inplace=True)
@@ -279,6 +279,11 @@ class SpectralBase:
             fast_transform : bool, optional
                 If True use fast transforms (if implemented), if
                 False use Vandermonde type
+            kind : str or functionspace, optional
+
+                - normal - use regular quadrature points
+                - uniform - use uniform mesh
+                - instance of SpectralBase - use quadrature mesh of this space
 
         Note
         ----
@@ -292,20 +297,18 @@ class SpectralBase:
         self._padding_backward(self.backward.input_array,
                                self.backward.tmp_array)
 
-        if kind == 'normal':
-            self._evaluate_expansion_all(self.backward.tmp_array,
-                                         self.backward.output_array,
-                                         fast_transform=fast_transform)
-        else:
-            if kind == 'uniform':
-                mesh = self.mesh(bcast=False, map_true_domain=False, uniform=True)
-            else:
-                mesh = kind.mesh()
-                if len(kind) > 1:
-                    mesh = np.squeeze(mesh[self.axis])
-            self._evaluate_expansion_all(self.backward.tmp_array,
-                                         self.backward.output_array,
-                                         x=mesh, fast_transform=False) # cannot use fast transforms on random mesh
+        mesh = None
+        if kind == 'uniform':
+            fast_transform = False
+            mesh = self.mesh(bcast=False, map_true_domain=False, uniform=True)
+        elif isinstance(kind, SpectralBase):
+            fast_transform = False
+            mesh = kind.mesh()
+            if len(kind) > 1:
+                mesh = np.squeeze(mesh[self.axis])
+        self._evaluate_expansion_all(self.backward.tmp_array,
+                                     self.backward.output_array,
+                                     x=mesh, fast_transform=fast_transform)
 
         if output_array is not None:
             output_array[...] = self.backward.output_array
@@ -518,6 +521,8 @@ class SpectralBase:
         output_array = self.scalar_product.tmp_array
         M = self.shape(False)
         weights = self.points_and_weights(M)[1]
+        if self.domain_factor() != 1:
+            weights /= self.domain_factor()
         P = self.evaluate_basis_all(argument=0)
         if input_array.ndim == 1:
             output_array[slice(0, M)] = np.dot(input_array*weights, np.conj(P))
@@ -687,8 +692,13 @@ class SpectralBase:
         raise NotImplementedError
 
     def sympy_reference_domain(self):
+        """Return reference domain sympified"""
         return self.reference_domain()
 
+    def domain_length(self):
+        """Return length of domain"""
+        return self.domain[1]-self.domain[0]
+
     def slice(self):
         """Return index set of current space"""
         return slice(0, self.N)
@@ -771,6 +781,12 @@ class SpectralBase:
     def is_orthogonal(self):
         return False
 
+    @property
+    def is_padded(self):
+        if self.padding_factor != 1:
+            return True
+        return False
+
     @property
     def is_composite_space(self):
         return 0
@@ -1242,6 +1258,23 @@ class MixedFunctionSpace:
         """Return dimension of scalar space"""
         return self.bases[0].dimensions
 
+    def slice(self):
+        return tuple(tuple([i]+[space.slice()]) for i, space in enumerate(self.flatten()))
+
+    def get_diagonal_axes(self):
+        return np.array([])
+
+    def get_ndiag_cum_dofs(self):
+        return np.array([0]+np.cumsum(self.dims()).tolist())
+
+    def get_ndiag_slices(self, j=()):
+        return np.array(self.slice())
+
+    def get_ndiag_slices_and_dims(self, j=()):
+        sl = self.get_ndiag_slices(j)
+        dims = self.get_ndiag_cum_dofs()
+        return sl, dims
+
 class VectorBasisTransform:
 
     __slots__ = ('_transforms',)
@@ -1344,6 +1377,7 @@ def inner_product(test, trial, measure=1):
             Basis is any of the classes from
 
             - :mod:`.chebyshev.bases`
+            - :mod:`.chebyshevu.bases`
             - :mod:`.legendre.bases`
             - :mod:`.fourier.bases`
             - :mod:`.laguerre.bases`
@@ -1375,7 +1409,7 @@ def inner_product(test, trial, measure=1):
     >>> [np.all(B[k] == v) for k, v in d.items()]
     [True, True, True]
     """
-    assert trial[0].__module__ == test[0].__module__
+    #assert trial[0].__module__ == test[0].__module__
     key = ((test[0].__class__, test[1]), (trial[0].__class__, trial[1]))
     sc = 1
     if measure != 1:
@@ -1392,13 +1426,14 @@ def inner_product(test, trial, measure=1):
             pass
         if measure.is_polynomial():
             measure = sp.simplify(measure)
+
         key = key + (test[0].domain, measure)
 
     mat = test[0]._get_mat()
     A = mat[key](test, trial)
     A.scale *= sc
-    if not test[0].domain_factor() == 1:
-        A.scale *= test[0].domain_factor()**(test[1]+trial[1])
+    #if not test[0].domain_factor() == 1:
+    #    A.scale *= test[0].domain_factor()**(test[1]+trial[1]-1)
     return A
 
 class FuncWrap:
