@@ -29,9 +29,9 @@ import types
 import numpy as np
 from shenfun import Function, TPMatrix, TrialFunction, TestFunction,\
     inner, la, Expr, CompositeSpace, BlockMatrix, extract_bc_matrices,\
-    SparseMatrix, get_simplified_tpmatrices
+    SparseMatrix, get_simplified_tpmatrices, ScipyMatrix
 
-__all__ = ('IRK3', 'RK4', 'ETDRK4', 'ETD')
+__all__ = ('IRK3', 'BackwardEuler', 'RK4', 'ETDRK4', 'ETD')
 
 #pylint: disable=unused-variable
 
@@ -82,7 +82,7 @@ class IntegratorBase:
         pass
 
     def solve(self, u, u_hat, dt, trange):
-        """Integrate forward in end_time
+        """Integrate forward in time
 
         Parameters
         ----------
@@ -124,12 +124,14 @@ class IRK3(IntegratorBase):
         self.dU1 = Function(T)
         self.a = (8./15., 5./12., 3./4.)
         self.b = (0.0, -17./60., -5./12.)
-        self.c = (0., 8./15., 2./3., 1)
+        self.c = (0.0, 8./15., 2./3., 1)
         self.solver = None
         self.bm = None
         self.rhs_mats = None
         self.w0 = Function(self.T)
-        self.mask = self.T.get_mask_nyquist()
+        self.mask = None
+        if hasattr(T, 'get_mask_nyquist'):
+            self.mask = T.get_mask_nyquist()
 
     def setup(self, dt):
         if isinstance(self.T, CompositeSpace):
@@ -140,16 +142,22 @@ class IRK3(IntegratorBase):
         v = TestFunction(self.T)
 
         # Note that we are here assembling implicit left hand side matrices,
-        # as well as matrices that can be used to assemble the right hande side
+        # as well as matrices that can be used to assemble the right hand side
         # much faster through matrix-vector products
 
         a, b = self.a, self.b
         self.solver = []
         self.rhs_mats = []
+        u0 = self.LinearRHS(u)
         for rk in range(3):
-            mats = inner(v, u - ((a[rk]+b[rk])*dt/2)*self.LinearRHS(u))
-            bc_mats = extract_bc_matrices([mats])
-            if self.T.tensor_rank == 0:
+            if u0:
+                mats = inner(v, u-((a[rk]+b[rk])*dt/2)*u0)
+            else:
+                mats = inner(v, u)
+            if self.T.dimensions == 1:
+                self.solver.append(la.Solver(mats))
+
+            elif self.T.tensor_rank == 0:
                 if len(mats[0].naxes) == 1:
                     self.solver.append(la.SolverGeneric1ND(mats))
                 elif len(mats[0].naxes) == 2:
@@ -157,22 +165,27 @@ class IRK3(IntegratorBase):
                 else:
                     raise NotImplementedError
             else:
-                self.solver.append(BlockMatrix(mats))
+                self.solver.append(BlockMatrixSolver(mats))
 
-            rhs_mats = inner(v, u + ((a[rk]+b[rk])*dt/2)*self.LinearRHS(u))
-            self.rhs_mats.append(BlockMatrix(rhs_mats+bc_mats))
-
-        self.mass = inner(u, v)
+            if u0:
+                rhs_mats = inner(v, u+((a[rk]+b[rk])*dt/2)*u0)
+            else:
+                rhs_mats = inner(v, u)
+            mat = ScipyMatrix if self.T.dimensions == 1 else BlockMatrix
+            self.rhs_mats.append(mat(rhs_mats))
 
     def compute_rhs(self, u, u_hat, dU, dU1, rk):
         a = self.a[rk]
         b = self.b[rk]
         dt = self.params['dt']
         dU = self.NonlinearRHS(u, u_hat, dU, **self.params)
-        dU.mask_nyquist(self.mask)
+        if self.mask:
+            dU.mask_nyquist(self.mask)
         w1 = dU*a*dt + dU1*b*dt
         dU1[:] = dU
-        return w1
+        if isinstance(dU, np.ndarray):
+            dU[:] = w1
+        return dU
 
     def solve(self, u, u_hat, dt, trange):
         if self.solver is None or abs(self.params['dt']-dt) > 1e-12:
@@ -182,15 +195,100 @@ class IRK3(IntegratorBase):
         while t < end_time-1e-8:
             self.tstep = tstep
             for rk in range(3):
+                self.params['ti'] = t+self.c[rk]*dt
                 dU = self.compute_rhs(u, u_hat, self.dU, self.dU1, rk)
                 dU += self.rhs_mats[rk].matvec(u_hat, self.w0)
                 u_hat = self.solver[rk](dU, u=u_hat)
-                u_hat.mask_nyquist(self.mask)
+                if self.mask:
+                    u_hat.mask_nyquist(self.mask)
 
             t += dt
             tstep += 1
             self.update(u, u_hat, t, tstep, **self.params)
 
+class BackwardEuler(IntegratorBase):
+    """First order backward Euler
+
+    Parameters
+    ----------
+        T : TensorProductSpace
+        L : function of TrialFunction(T)
+            To compute linear part of right hand side
+        N : function
+            To compute nonlinear part of right hand side
+        update : function
+            To be called at the end of a timestep
+        params : dictionary
+            Any relevant keyword arguments
+
+    """
+    def __init__(self, T,
+                 L=None,
+                 N=None,
+                 update=None,
+                 **params):
+        IntegratorBase.__init__(self, T, L=L, N=N, update=update, **params)
+        self.dU = Function(T)
+        self.dU1 = Function(T)
+        self.solver = None
+        self.rhs_mats = None
+        self.w0 = Function(self.T)
+        self.mask = None
+        if hasattr(T, 'get_mask_nyquist'):
+            self.mask = T.get_mask_nyquist()
+
+    def setup(self, dt):
+        if isinstance(self.T, CompositeSpace):
+            assert self.T.tensor_rank > 0, 'BackwardEuler only works for tensors, not generic CompositeSpaces'
+
+        self.params['dt'] = dt
+        u = TrialFunction(self.T)
+        v = TestFunction(self.T)
+        mats = inner(u-dt*self.LinearRHS(u), v)
+        M = inner(u, v)
+
+        if self.T.dimensions == 1:
+            self.solver = la.Solve(mats)
+            self.rhs_mats = M
+            return
+
+        if self.T.tensor_rank == 0:
+            if len(mats[0].naxes) == 1:
+                self.solver = la.SolverGeneric1ND(mats)
+            elif len(mats[0].naxes) == 2:
+                self.solver = la.SolverGeneric2ND(mats)
+            else:
+                raise NotImplementedError
+        else:
+            self.solver = BlockMatrixSolver(mats)
+        self.rhs_mats = BlockMatrix(M if isinstance(M, list) else [M])
+
+    def compute_rhs(self, u, u_hat, dU, dU1):
+        dt = self.params['dt']
+        dU = self.NonlinearRHS(u, u_hat, dU, **self.params)
+        if self.mask:
+            dU.mask_nyquist(self.mask)
+        w1 = dU*2*dt - dU1*dt
+        dU1[:] = dU
+        return w1
+
+    def solve(self, u, u_hat, dt, trange):
+        if self.solver is None or abs(self.params['dt']-dt) > 1e-12:
+            self.setup(dt)
+        t, end_time = trange
+        tstep = self.tstep = 0
+        while t < end_time-1e-8:
+            self.params['ti'] = t
+            self.tstep = tstep
+            dU = self.compute_rhs(u, u_hat, self.dU, self.dU1)
+            dU += self.rhs_mats.matvec(u_hat, self.w0)
+            u_hat = self.solver(dU, u=u_hat)
+            if self.mask:
+                u_hat.mask_nyquist(self.mask)
+            t += dt
+            tstep += 1
+            self.update(u, u_hat, t, tstep, **self.params)
+
 
 class ETD(IntegratorBase):
     """Exponential time differencing Euler method
